require 'rest_client'

# Version
require 'foxycart/version'

# API operations
require 'foxycart/api_operations/create'
# require 'foxycart/api_operations/update'
# require 'foxycart/api_operations/delete'
# require 'foxycart/api_operations/list'

# Resources
require 'foxycart/store'

# Errors
require 'foxycart/errors/foxycart_error'
require 'foxycart/errors/authentication_error'

module FoxyCart
  @api_base = 'https://api-sandbox.foxycart.com'

  class << self
    attr_accessor :api_key
  end

  def self.api_url(url='')
    @api_base + url
  end

  def self.request_headers(api_key)
    {
      'FOXYCART-API-VERSION' => '1', 
      'Authorization' => "Bearer #{api_key}",
    }
  end

  def self.request(method, url, api_key, params={}, headers={})
    unless api_key ||= @api_key
      raise AuthenticationError.new('No API key provided. ' +
        'Set your API key using "FoxyCart.api_key = <API-KEY>".')
    end

    url = api_url(url)

    case method.to_s.downcase.to_sym
    when :get, :head, :delete
      # Make params into GET parameters
      url += "#{URI.parse(url).query ? '&' : '?'}#{uri_encode(params)}" if params && params.any?
      payload = nil
    else
      payload = uri_encode(params)
    end

    # post(url, data: post_data, headers: FoxyCart.headers)
    request_opts = {
      :headers => request_headers(api_key).update(headers),
      :method => method, 
      :open_timeout => 30,
      :payload => payload, 
      :url => url, 
      :timeout => 80
    }

    execute_request(request_opts)
  end

private

  def self.uri_encode(params)
    Util.flatten_params(params).
      map { |k,v| "#{k}=#{Util.url_encode(v)}" }.join('&')
  end

  def self.execute_request(opts)
    RestClient::Request.execute(opts)
  end

end
